---
title: Templating
description: Learn how Benthos templates work.
---

<!--
     THIS FILE IS AUTOGENERATED!

     To make changes please edit the contents of:
     internal/template/docs.md.tmpl
-->

EXPERIMENTAL: Templates are an experimental feature and therefore subject to change outside of major version releases.

Templates are a way to define new Benthos components (similar to plugins) that are implemented by generating a Benthos config snippet from pre-defined parameter fields. This is useful when a common pattern of Benthos configuration is used but with varying parameters each time.

A template is defined in a YAML file that can be imported when Benthos runs using the flag `-t`:

```sh
benthos -t "./templates/*.yaml" -c ./config.yaml
```

The template describes the type of the component and configuration fields that can be used to customize it, followed by a [Bloblang mapping][bloblang.about] that translates an object containing those fields into a benthos config structure. This allows you to use logic to generate more complex configurations:

import Tabs from '@theme/Tabs';

<Tabs defaultValue="template" values={[
  { label: 'Template', value: 'template', },
  { label: 'Config', value: 'config', },
  { label: 'Result', value: 'result', },
]}>

import TabItem from '@theme/TabItem';

<TabItem value="template">

```yml
name: aws_sqs_list
type: input

fields:
  - name: urls
    type: string
    kind: list
  - name: region
    type: string
    default: us-east-1

mapping: |
  root.broker.inputs = this.urls.map_each(url -> {
    "aws_sqs": {
      "url": url,
      "region": this.region,
    }
  })
```

</TabItem>
<TabItem value="config">

```yml
input:
  aws_sqs_list:
    urls:
      - https://sqs.us-east-2.amazonaws.com/123456789012/MyQueue1
      - https://sqs.us-east-2.amazonaws.com/123456789012/MyQueue2

pipeline:
  processors:
    - bloblang: |
        root.id = uuid_v4()
        root.foo = this.inner.foo
        root.body = this.outter
```

</TabItem>
<TabItem value="result">

```yaml
input:
  broker:
    inputs:
      - aws_sqs:
          url: https://sqs.us-east-2.amazonaws.com/123456789012/MyQueue1
          region: us-east-1
      - aws_sqs:
          url: https://sqs.us-east-2.amazonaws.com/123456789012/MyQueue2
          region: us-east-1

pipeline:
  processors:
    - bloblang: |
        root.id = uuid_v4()
        root.foo = this.inner.foo
        root.body = this.outter
```

</TabItem>

</Tabs>

You can see more examples of templates, including some that are included as part of the standard Benthos distribution, at [https://github.com/Jeffail/benthos/tree/master/template](https://github.com/Jeffail/benthos/tree/master/template).

## Fields

The schema of a template file is as follows:

{{template "field_docs" . -}}

[bloblang.about]: /docs/guides/bloblang/about
